/*****************************************************************************
 * ugBASIC - an isomorphic BASIC language compiler for retrocomputers        *
 *****************************************************************************
 * Copyright 2021-2022 Marco Spedaletti (asimov@mclink.it)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *----------------------------------------------------------------------------
 * Concesso in licenza secondo i termini della Licenza Apache, versione 2.0
 * (la "Licenza"); è proibito usare questo file se non in conformità alla
 * Licenza. Una copia della Licenza è disponibile all'indirizzo:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Se non richiesto dalla legislazione vigente o concordato per iscritto,
 * il software distribuito nei termini della Licenza è distribuito
 * "COSÌ COM'È", SENZA GARANZIE O CONDIZIONI DI ALCUN TIPO, esplicite o
 * implicite. Consultare la Licenza per il testo specifico che regola le
 * autorizzazioni e le limitazioni previste dalla medesima.
 ****************************************************************************/

/****************************************************************************
 * INCLUDE SECTION 
 ****************************************************************************/

#include "../../ugbc.h"

/****************************************************************************
 * CODE SECTION 
 ****************************************************************************/

/**
 * @brief Emit code for <strong>KILL ...</strong>
 * 
 * @param _environment Current calling environment
 * @param _handle Identifier of the thread
 */
/* <usermanual>
@keyword KILL

@english
This keyword will kill a (parallel) procedure. 

@italian
Questa parola chiave termina una procedura (parallela).

@syntax KILL [param]

@example KILL handleThread

@target all
</usermanual> */
void kill_procedure( Environment * _environment, char * _handle ) {

    MAKE_LABEL

    Variable * threadId = variable_retrieve( _environment, _handle );

    if ( threadId->type != VT_THREAD ) {
        CRITICAL_CANNOT_KILL_NOT_THREADID( _handle );
    }
    
    _environment->anyProtothread = 1;

    cpu_protothread_unregister( _environment, threadId->realName );

}

